/*
 * SBS620
 * PCI-VME
 * 
 * H.Baba (Rikkyo Univ.)
 * Jun 26, 2002
 */

#include "sbs620.h"

int sbs620_init_register();
void sbs620_set_amsr(unsigned char am);
void sbs620_vread16(unsigned long addr,short *data);
void sbs620_vread32(unsigned long addr,long *data);
void sbs620_vwrite16(unsigned long addr,short *data);
void sbs620_vwrite32(unsigned long addr,long *data);
void sbs620_dma_vread32_start(unsigned long addr);
int sbs620_dma_vread32_store(long *data);
int sbs620_check_dma(void);
void sbs620_init_dma(unsigned long addr,int rw,int ls);
void sbs620_define_intlevel(int level);
void sbs620_enable_interrupt(void);
void sbs620_disable_interrupt(void);
int sbs620_check_interrupt(void);
int sbs620_read_intvector(void);

struct pci_config {
    unsigned addr[4];
    unsigned char irq;
}sbsconfig[16];

struct vpci_reg{
  char *io;
  char *csr;
  char *vme;
  char *pci;
  char *dma;
  char *remote;
  char *dma_buff;
}vreg[16];


static int amsr[16];
static unsigned long base_addr[16],offset_addr[16];
volatile static int intlevel[16],devicen;
volatile static short intleveln = 0;

int sbs620_init_register(int dn){
  unsigned char sta;
  unsigned bus_addr;

  vreg[dn].csr = ioremap(sbsconfig[dn].addr[1],SBS620_CSR_SIZE);
  vreg[dn].vme = ioremap(sbsconfig[dn].addr[2],SBS620_MAP_SIZE);
  vreg[dn].pci = vreg[dn].vme + SBS620_PCI_OFFSET;
  vreg[dn].dma = vreg[dn].vme + SBS620_DMA_OFFSET;
  vreg[dn].remote = ioremap(sbsconfig[dn].addr[3],SBS620_REMOTE_SIZE);

  vreg[dn].dma_buff = kmalloc(SBS620_DMA_SIZE,GFP_KERNEL|GFP_DMA);
  if(vreg[dn].dma_buff <= 0){
    printk("Can't DAM Buffer was allocated.\n");
    return -1;
  }
  bus_addr = virt_to_bus(vreg[dn].dma_buff) & SBS620_DMA_BASE_MASK;
  writel(bus_addr,vreg[dn].dma);

  sta = readb(vreg[dn].csr+SBS620_LOCAL_STATUS) & SBS620_POWER;
 
  if(sta > 0){
    printk("SBS620 is power off or cable is not connected.\n");
    /* return -1; */
  }
  
  sbs620_set_amsr(A32);
  intlevel[dn] = 0;

  return 0;
}

void sbs620_set_amsr(unsigned char am){
  amsr[0] = ((am << SBS620_AM_SHIFT) | 
	     (SBS620_BUS_IO << SBS620_FUNC_SHIFT)|
	     (SBS620_SWAP_NONE << SBS620_SWAP_SHIFT));
}

void sbs620_set_vmeaddr(unsigned long addr){
  base_addr[0] = addr & SBS620_BASE_MASK;
  writel(base_addr[0] | amsr[0], vreg[0].vme);
}

void sbs620_vread16(unsigned long addr,short *data){

  offset_addr[0] = addr & SBS620_OFFSET_MASK;
  sbs620_set_vmeaddr(addr);

  *data = readw(vreg[0].remote + offset_addr[0]);
}

void sbs620_vread32(unsigned long addr,long *data){

  offset_addr[0] = addr & SBS620_OFFSET_MASK;
  sbs620_set_vmeaddr(addr);

  *data = readl(vreg[0].remote + offset_addr[0]);
}

void sbs620_vwrite16(unsigned long addr,short *data){
  offset_addr[0] = addr & SBS620_OFFSET_MASK;
  sbs620_set_vmeaddr(addr);

  writew(*data,vreg[0].remote + offset_addr[0]);

}

void sbs620_vwrite32(unsigned long addr,long *data){
  offset_addr[0] = addr & SBS620_OFFSET_MASK;
  sbs620_set_vmeaddr(addr);

  writel(*data,vreg[0].remote + offset_addr[0]);

}

void sbs620_dma_vread32(unsigned long addr,long *data){
  offset_addr[0] = addr & SBS620_OFFSET_MASK;
  sbs620_set_vmeaddr(addr);
  
}

void sbs620_dma_vread32_start(unsigned long addr){
  sbs620_init_dma(addr,SBS620_DMA_READ,SBS620_DMA_LONGWORD);
}

int sbs620_dma_vread32_store(long *data){
  int ret;

  if(sbs620_check_dma()){
    ret = 1;
  }else{
    ret = 0;
  }

  writeb(0,vreg[0].csr + SBS620_DMA_COMMAND);

  return ret;
}

int sbs620_check_dma(void){
  return readb(vreg[0].csr + SBS620_DMA_COMMAND) & SBS620_DMA_DONE;
}

void sbs620_init_dma(unsigned long addr,int rw,int ls){
  unsigned com,reg,remainder_cnt = 0,packet_cnt = 0;

  com = rw | ls;
  writeb(com,vreg[0].csr + SBS620_DMA_COMMAND);

  reg = 0;
  writeb(reg,vreg[0].csr + SBS620_DMA_PCI_ADDR_2_7);
  writeb(reg,vreg[0].csr + SBS620_DMA_PCI_ADDR_8_15);
  writeb(reg,vreg[0].csr + SBS620_DMA_PCI_ADDR_16_23);

  writeb((addr) & 0x000000ff,vreg[0].csr + SBS620_DMA_VME_ADDR_0_7);
  writeb((addr >> 8) & 0x000000ff,vreg[0].csr + SBS620_DMA_VME_ADDR_8_15);
  writeb((addr >> 16) & 0x000000ff,vreg[0].csr + SBS620_DMA_VME_ADDR_16_23);
  writeb((addr >> 24) & 0x000000ff,vreg[0].csr + SBS620_DMA_VME_ADDR_24_31);

  remainder_cnt = SBS620_DMA_SIZE % SBS620_DMA_PACKET_SIZE;
  packet_cnt = SBS620_DMA_SIZE / SBS620_DMA_PACKET_SIZE;
  writeb(remainder_cnt,vreg[0].csr + SBS620_DMA_REMOTE_REMAINDER_CNT);
  writeb(remainder_cnt,vreg[0].csr + SBS620_DMA_REMAINDER_CNT);
  writeb((packet_cnt) & 0x00ff,vreg[0].csr + SBS620_DMA_PACKET_CNT_0_7);
  writeb((packet_cnt >> 8) & 0x00ff,vreg[0].csr + SBS620_DMA_PACKET_CNT_8_15);

  reg = SBS620_DMA_BLOCK_MODE | SBS620_DMA_DIS_INT_PASS;
  writeb(reg,vreg[0].csr + SBS620_REMOTE_COMMAND2);

  com |= SBS620_DMA_STARTDMA;
  writeb(com,vreg[0].csr + SBS620_DMA_COMMAND);
}

void sbs620_define_intlevel(int level){
  intleveln = (short)level;
  intlevel[0] = 1<<level;
}

void sbs620_enable_interrupt(void){
  writeb(SBS620_CLEAR_INTERRUPT,vreg[0].csr + SBS620_LOCAL_COMMAND);
  writeb(SBS620_ENABLE_INTERRUPT,vreg[0].csr + SBS620_INT_CTRL);
}
  
void sbs620_disable_interrupt(void){
  writeb(SBS620_DISABLE_INTERRUPT,vreg[0].csr + SBS620_INT_CTRL);
  writeb(SBS620_CLEAR_INTERRUPT,vreg[0].csr + SBS620_LOCAL_COMMAND);
}

int sbs620_check_interrupt(void){
  if((readb(vreg[0].csr + SBS620_INT_CTRL) & SBS620_IS_INTERRUPT)
     == SBS620_IS_INTERRUPT){
    if((readb(vreg[0].csr + SBS620_INT_STATUS) & intlevel[0]) == intlevel[0]){
      return 1;
    }
  }

  return 0;
}

int sbs620_read_intvector(void){
  writew(intleveln,vreg[0].csr + SBS620_REMOTE_COMMAND1);
  return readw(vreg[0].csr + SBS620_IACK_LOW);
}
