/* nbbqccio.c
   CAMAC I/O Interface Driver

   last modified : 03/08/19 20:12:04 
*/


#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/ioctl.h>
#include <linux/mm.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/config.h>
#include <linux/pci.h>
#include <linux/errno.h>
#include <linux/poll.h>
#include <asm/io.h>
#include <asm/uaccess.h>

#include "common.h"
#include "ccioctrl.h"
#include "nbbqccio.h"

#if LINUX_VERSION_CODE >=  0x020410
  MODULE_LICENSE("GPL");
#endif

static int nbbqccio_major = NBBQCCIO_MAJOR;
static const char *nbbqccio_name = "nbbqccio";

ssize_t nbbqccio_read(struct file *file,char *buff,size_t count,loff_t *pos);
static int nbbqccio_open(struct inode* inode, struct file* filep);
static int nbbqccio_release(struct inode* inode, struct file* filep);
static int nbbqccio_ioctl(struct inode *inode, struct file *filep,
			 unsigned int cmd, unsigned long arg);
static unsigned int nbbqccio_poll(struct file *file,poll_table *wait);


static int girq,chkbuff,chkblk;
#if LINUX_VERSION_CODE >=  0x020400
wait_queue_head_t nbbqccio_wait_queue;
#else
struct wait_queue *nbbqccio_wait_queue = NULL;
#endif

#if LINUX_VERSION_CODE >=  0x020400
static struct file_operations nbbqccio_fops = {
  read: nbbqccio_read,
  poll: nbbqccio_poll,
  ioctl: nbbqccio_ioctl,
  open: nbbqccio_open,
  release: nbbqccio_release,
};
#else
static struct file_operations nbbqccio_fops = {
  NULL,             // loff_t  llseek
  nbbqccio_read,     // ssize_t read
  NULL,             // ssize_t write
  NULL,             // int     readdir
  nbbqccio_poll,     // uint    poll
  nbbqccio_ioctl,    // int     ioctl
  NULL,             // int     mmap
  nbbqccio_open,     // int     open
  NULL,             // int     flush
  nbbqccio_release,  // int     release
};
#endif



int init_module(void){
  int ret;

  ret = register_chrdev(nbbqccio_major,nbbqccio_name,&nbbqccio_fops);
  if(ret < 0){
    printk("%s : can't regist.\n",nbbqccio_name);
    return ret;
  }
  if(nbbqccio_major == 0) {
    nbbqccio_major = ret;
  }

  girq = get_irq();
  chkbuff = 0;

#if LINUX_VERSION_CODE >=  0x020400
  init_waitqueue_head(&nbbqccio_wait_queue);
#endif

  printk("%s : nbbqccio was installed (irq %d).\n",nbbqccio_name,girq);

  return 0;
}

void cleanup_module(void){
  unregister_chrdev(nbbqccio_major,nbbqccio_name);
  printk("%s: nbbqccio was unregistered.\n", nbbqccio_name);
}

static int nbbqccio_open(struct inode* inode, struct file* filep){

  if(MOD_IN_USE){
    return -EBUSY;
  }

  chkblk = 0;


  MOD_INC_USE_COUNT;

  return 0;
}

static int nbbqccio_release(struct inode* inode, struct file* filep){

  MOD_DEC_USE_COUNT;

  return 0;
}

ssize_t nbbqccio_read(struct file *file,char *buff,size_t count,loff_t *pos){

  /* copy_to_user(buff,data,sizeof(data)); */

  return 0;
}


static int nbbqccio_ioctl(struct inode* inode, struct file *filep,
			 unsigned int cmd, unsigned long arg){

  unsigned short sval=0,ssval[2];
  unsigned long lval=0, cnaf=0;

  switch(cmd){
  case NBBQCCIO_CTRL:
    /* int cnaf */
    copy_from_user(&cnaf,(void *)arg,4);
#ifdef CCIO_K2915
    cnaf += 0x00000200;
#endif
    control(cnaf);
    break;
  case NBBQCCIO_READ16:
    /* int cnaf, ushort data */
    copy_from_user(&cnaf,(void *)arg,4);
#ifdef CCIO_K2915
    cnaf += 0x00000200;
#endif
    read16(cnaf,&sval);
    copy_to_user((void *)arg+4,&sval,2);
    break;
  case NBBQCCIO_READ24:
    /* int cnaf, ulong data */
    copy_from_user(&cnaf,(void *)arg,4);
#ifdef CCIO_K2915
    cnaf += 0x00000200;
#endif
    read24(cnaf,ssval);
    copy_to_user((void *)arg+4,ssval,4);
    break;
  case NBBQCCIO_WRITE16:
    /* ulong addr, ushort data */
    copy_from_user(&cnaf,(void *)arg,4);
    copy_from_user(&sval,(void *)arg+4,2);
#ifdef CCIO_K2915
    cnaf += 0x00000200;
#endif
    write16(cnaf,&sval);
    break;
  case NBBQCCIO_WRITE24:
    /* ulong addr, ulong data */
    copy_from_user(&cnaf,(void *)arg,4);
    copy_from_user(&lval,(void *)arg+4,4);
#ifdef CCIO_K2915
    cnaf += 0x00000200;
#endif
    write24(cnaf,(int *)&lval);
    break;
  case NBBQCCIO_Z:
    copy_from_user(&cnaf,(void *)arg,4);
    crate_z(cnaf);
    break;
  case NBBQCCIO_C:
    copy_from_user(&cnaf,(void *)arg,4);
    crate_c(cnaf);
    break;
  case NBBQCCIO_SETI:
    copy_from_user(&cnaf,(void *)arg,4);
    crate_seti(cnaf);
    break;
  case NBBQCCIO_DELI:
    copy_from_user(&cnaf,(void *)arg,4);
    crate_deli(cnaf);
    break;
  }

  return 1;
}

static unsigned int nbbqccio_poll(struct file *file,poll_table *wait){
  poll_wait(file,&nbbqccio_wait_queue,wait);
  if(chkbuff == 1){
    return POLLIN;
  }else{
    return 0;
  }
}


