import java.awt.*;
import java.awt.event.*;

public class Twodim extends Frame
	implements WindowListener,ItemListener{
	private TwodimCanvas twodimcanvas;
    private int dimNumber,interval,size,grid;
    private boolean clearflag;
    private DimScaler scaler;
    private Choice intervalChoice,gridChoice;
    Label xLabel,yLabel,countLabel;
    Panel pointPanel;

	public Twodim(int num,int inter,int ss){
        dimNumber = num;
        interval = inter;
        size = ss;
        this.setTitle(StartupDAQ.conFrame.dimdata[dimNumber].getName());
        /* Choice */
        intervalChoice = new Choice();
        intervalChoice.add("50");
        intervalChoice.add("100");
        intervalChoice.add("200");
        intervalChoice.add("500");
        intervalChoice.add("1000");
        intervalChoice.add("NLog");
        intervalChoice.add("Log");
        intervalChoice.select(interval);
        intervalChoice.addItemListener(this);

        grid = 128;
        gridChoice = new Choice();
        gridChoice.add("None");
        gridChoice.add("16");
        gridChoice.add("32");
        gridChoice.add("64");
        gridChoice.add("128");
        gridChoice.add("256");
        gridChoice.select(4);
        gridChoice.addItemListener(this);

        scaler = new DimScaler(interval);
        pointPanel = new Panel();
		twodimcanvas = new TwodimCanvas();
        xLabel = new Label("X :   0",Label.CENTER);
        yLabel = new Label("Y :   0",Label.CENTER);
        countLabel = new Label("Count :   0",Label.CENTER);
        pointPanel.add(xLabel);
        pointPanel.add(yLabel);
        pointPanel.add(countLabel);

        if(size == 512){
            this.setSize(600,580);
            twodimcanvas.setBounds(35,25,512,512);
            pointPanel.setBounds(60,540,300,20);
            scaler.setBounds(550,387,50,150);
            intervalChoice.setBounds(550,364,50,10);
            gridChoice.setBounds(550,344,50,10);
        }else if(size == 384){
	  this.setSize(472,452);
	  twodimcanvas.setBounds(35,25,384,384);
	  pointPanel.setBounds(60,412,300,20);
	  scaler.setBounds(422,259,50,150);
	  intervalChoice.setBounds(422,236,50,10);
	  gridChoice.setBounds(422,216,50,10);
	}else{
            this.setSize(344,324);
            twodimcanvas.setBounds(35,25,256,256);
            pointPanel.setBounds(60,284,200,20);
            scaler.setBounds(294,131,50,150);
            intervalChoice.setBounds(294,111,50,10);
            gridChoice.setBounds(294,91,50,10);
        }
        this.setFont(StartupDAQ.defaultFont);
        this.setBackground(Color.lightGray);
        this.setLayout(null);
        addWindowListener(this);

        this.add(twodimcanvas);
        this.add(pointPanel);
        this.add(scaler);
        this.add(intervalChoice);
        this.add(gridChoice);
	}

    public void setPointLabel(int x,int y){
        xLabel.setText("X : "+Integer.toString(x));
        yLabel.setText("y : "+Integer.toString(y));
        countLabel.setText("Count : "+Integer.toString(StartupDAQ.conFrame.dimdata[dimNumber].read(x,y)));
    }

    public void setClearFlag(){
        clearflag = true;
    }

    public void repaintcanvas(){
        this.twodimcanvas.repaint();
    }

    public void paint(Graphics g){
        if(size == 512){
            g.drawLine(34,25,34,537);
            g.drawLine(34,537,547,537);
            g.drawString("512",15,35);
            g.drawString("512",532,549);
        }else if(size == 384){
            g.drawLine(34,25,34,409);
            g.drawLine(34,409,419,409);
            g.drawString("384",15,35);
            g.drawString("384",404,421);
        }else{
            g.drawLine(34,25,34,281);
            g.drawLine(34,281,291,281);
            g.drawString("256",15,35);
            g.drawString("256",276,293);
        }
    }

	class TwodimCanvas extends Canvas implements MouseMotionListener{

		public TwodimCanvas(){
            setBackground(Color.white);
            addMouseMotionListener(this);
		}

		public void paint(Graphics g){
        g.setColor(Color.lightGray);
        if(grid != 0){
            for(int i=1;i<size/grid;i++){
                g.drawLine(0,grid*i,size,grid*i);
                g.drawLine(grid*i,0,grid*i,size);
            }
        }
        update(g);
		}

		public void update(Graphics g){
            if(clearflag == true){
                g.clearRect(0,0,size,size);
                g.setColor(Color.lightGray);
                if(grid != 0){
                    for(int i=1;i<size/grid;i++){
                        g.drawLine(0,grid*i,size,grid*i);
                        g.drawLine(grid*i,0,grid*i,size);
                    }
                }
                clearflag = false;
            }
            for(int i=0;i<size;i++){
                for(int j=0;j<size;j++){
                    if(StartupDAQ.conFrame.dimdata[dimNumber].read(i,j) > 0){
                        int sw = StartupDAQ.conFrame.dimdata[dimNumber].read(i,j);
                        if(sw <= scaler.dg){
                            g.setColor(Color.darkGray);
                        }else if(sw <= scaler.b){
                            g.setColor(Color.blue);
                        }else if(sw <= scaler.c){
                            g.setColor(Color.cyan);
                        }else if(sw <= scaler.g){
                            g.setColor(Color.green);
                        }else if(sw <= scaler.o){
                            g.setColor(Color.orange);
                        }else if(sw <= scaler.p){
                            g.setColor(Color.pink);
                        }else{
                            g.setColor(Color.red);
                        }
                        g.fillRect(i,size-j,1,1);
                    }
                }
            }
		}

        public void mouseDragged(MouseEvent e){
        }

        public void mouseMoved(MouseEvent e){
            setPointLabel(e.getX(),size-e.getY());
        }
	}

    void setGrid(int gr){
        int[] grint = {0,16,32,64,128,256};
        grid = grint[gr];
    }

    /* Twodim ItemListener */
    public void itemStateChanged(ItemEvent evt){
        Object obj = evt.getSource();
        if(obj == intervalChoice){
            interval = intervalChoice.getSelectedIndex();
            scaler.setSize(interval);
            repaintcanvas();
        }else if(obj == gridChoice){
            setGrid(gridChoice.getSelectedIndex());
            clearflag = true;
            repaintcanvas();
        }
    }

	/* Twodim WindowLintener */
	public void windowClosed(WindowEvent evt){
	}

	public void windowClosing(WindowEvent evt){
		dispose();
	}

	public void windowOpened(WindowEvent evt){
	}

	public void windowIconified(WindowEvent evt){
	}

	public void windowDeiconified(WindowEvent evt){
	}

	public void windowActivated(WindowEvent evt){
	}

	public void windowDeactivated(WindowEvent evt){
	}
}
