/* nbbqvio.c
   VME I/O Interface Driver
   H.Baba
   last modified : 03/05/02 13:29:18 
*/


#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/ioctl.h>
#include <linux/mm.h>
#include <linux/fs.h>
#include <linux/sched.h>
#include <linux/config.h>
#include <linux/pci.h>
#include <linux/errno.h>
#include <linux/poll.h>
#include <asm/io.h>
#include <asm/uaccess.h>

#include "common.h"
#include "vioctrl.h"
#include "nbbqvio.h"

#if LINUX_VERSION_CODE >=  0x020410
MODULE_LICENSE("GPL");
#endif

static int nbbqvio_major = NBBQVIO_MAJOR;
static const char *nbbqvio_name = "nbbqvio";
//static char *irq_name = "irq_nbbqvio";

ssize_t nbbqvio_read(struct file *file,char *buff,size_t count,loff_t *pos);
static int nbbqvio_open(struct inode* inode, struct file* filep);
static int nbbqvio_release(struct inode* inode, struct file* filep);
static int nbbqvio_ioctl(struct inode *inode, struct file *filep,
			 unsigned int cmd, unsigned long arg);
static unsigned int nbbqvio_poll(struct file *file,poll_table *wait);
//static void nbbqvio_interrupt(int irq,void *dev_id,struct pt_regs* regs);


static int girq,chkbuff,chkblk;
#if LINUX_VERSION_CODE >=  0x020400
wait_queue_head_t nbbqvio_wait_queue;
#else
struct wait_queue *nbbqvio_wait_queue = NULL;
#endif

#if LINUX_VERSION_CODE >=  0x020400
static struct file_operations nbbqvio_fops = {
  read: nbbqvio_read,
  poll: nbbqvio_poll,
  ioctl: nbbqvio_ioctl,
  open: nbbqvio_open,
  release: nbbqvio_release,
};
#else
static struct file_operations nbbqvio_fops = {
  NULL,             // loff_t  llseek
  nbbqvio_read,     // ssize_t read
  NULL,             // ssize_t write
  NULL,             // int     readdir
  nbbqvio_poll,     // uint    poll
  nbbqvio_ioctl,    // int     ioctl
  NULL,             // int     mmap
  nbbqvio_open,     // int     open
  NULL,             // int     flush
  nbbqvio_release,  // int     release
};
#endif

int init_module(void){
  int ret;

  ret = register_chrdev(nbbqvio_major,nbbqvio_name,&nbbqvio_fops);
  if(ret < 0){
    printk("%s : can't regist.\n",nbbqvio_name);
    return ret;
  }
  if(nbbqvio_major == 0) {
    nbbqvio_major = ret;
  }

  girq = get_irq();
  chkbuff = 0;

#if LINUX_VERSION_CODE >=  0x020400
  init_waitqueue_head(&nbbqvio_wait_queue);
#endif

  printk("%s : nbbqvio was installed (irq %d).\n",nbbqvio_name,girq);

  return 0;
}

void cleanup_module(void){
  unregister_chrdev(nbbqvio_major,nbbqvio_name);
  printk("%s: nbbqvio was unregistered.\n", nbbqvio_name);
}

static int nbbqvio_open(struct inode* inode, struct file* filep){

  if(MOD_IN_USE){
    return -EBUSY;
  }

  chkblk = 0;

  /* ret = request_irq(girq,nbbqvio_interrupt,SA_INTERRUPT,nbbqvio_name,irq_name); */

  MOD_INC_USE_COUNT;

  return 0;
}

static int nbbqvio_release(struct inode* inode, struct file* filep){

  MOD_DEC_USE_COUNT;

  return 0;
}

ssize_t nbbqvio_read(struct file *file,char *buff,size_t count,loff_t *pos){

  /* copy_to_user(buff,data,sizeof(data)); */

  return 1;
}


static int nbbqvio_ioctl(struct inode* inode, struct file *filep,
			 unsigned int cmd, unsigned long arg){

  unsigned short sval=0;
  unsigned long addr, lval=0;

  switch(cmd){
  case NBBQVIO_AMSR:
    /* int am */
    copy_from_user(&lval,(void *)arg,4);
    set_amsr((char)lval);
    break;
  case NBBQVIO_READ16:
    /* ulong addr, ushort data */
    copy_from_user(&addr,(void *)arg,4);
    vread16(addr,&sval);
    copy_to_user((void *)arg+4,&sval,2);
    break;
  case NBBQVIO_READ32:
    /* ulong addr, ulong data */
    copy_from_user(&addr,(void *)arg,4);
    vread32(addr,&lval);
    copy_to_user((void *)arg+4,&lval,4);
    break;
  case NBBQVIO_WRITE16:
    /* ulong addr, ushort data */
    copy_from_user(&addr,(void *)arg,4);
    copy_from_user(&sval,(void *)arg+4,2);
    vwrite16(addr,&sval);
    break;
  case NBBQVIO_WRITE32:
    /* ulong addr, ulong data */
    copy_from_user(&addr,(void *)arg,4);
    copy_from_user(&lval,(void *)arg+4,4);
    vwrite32(addr,&lval);
    break;
  case NBBQVIO_DMAR16:
    break;
  case NBBQVIO_DMAR32:
    break;
  case NBBQVIO_DMAW16:
    break;
  case NBBQVIO_DMAW32:
    break;
  }

  return 1;
}

static unsigned int nbbqvio_poll(struct file *file,poll_table *wait){
  poll_wait(file,&nbbqvio_wait_queue,wait);
  if(chkbuff == 1){
    return POLLIN;
  }else{
    return 0;
  }
}

/*
  static void nbbqvio_interrupt(int irq, void* dev_id, struct pt_regs* regs){
  wake_up_interruptible(&nbbqvio_wait_queue);
  }
*/


